'use strict';

const index = require('./eslint-plugin-command.5967170a.cjs');

const version = "0.1.3";

function RuleCreator(urlCreator) {
  return function createNamedRule({
    name,
    meta,
    ...rule
  }) {
    return createRule({
      meta: {
        ...meta,
        docs: {
          ...meta.docs,
          url: urlCreator(name)
        }
      },
      ...rule
    });
  };
}
function createRule({
  create,
  defaultOptions,
  meta
}) {
  return {
    create: (context) => {
      const optionsWithDefault = context.options.map((options, index) => {
        return {
          ...defaultOptions[index] || {},
          ...options || {}
        };
      });
      return create(context, optionsWithDefault);
    },
    defaultOptions,
    meta
  };
}
const createEslintRule = RuleCreator(
  () => "https://github.com/antfu/eslint-plugin-command"
);

const SKIP = Symbol("skip");
const STOP = Symbol("stop");
function traverse(context, node, visitor) {
  const allVisitorKeys = context.sourceCode.visitorKeys;
  const queue = [];
  queue.push({
    node,
    parent: null,
    parentKey: null,
    parentPath: null
  });
  while (queue.length) {
    const currentPath = queue.shift();
    const result = visitor(currentPath, { SKIP, STOP });
    if (result === STOP)
      return false;
    if (result === SKIP)
      continue;
    const visitorKeys = allVisitorKeys[currentPath.node.type];
    if (!visitorKeys)
      continue;
    for (const visitorKey of visitorKeys) {
      const child = currentPath.node[visitorKey];
      if (!child) {
        continue;
      } else if (Array.isArray(child)) {
        for (const item of child) {
          queue.push({
            node: item,
            parent: currentPath.node,
            parentKey: visitorKey,
            parentPath: currentPath
          });
        }
      } else {
        queue.push({
          node: child,
          parent: currentPath.node,
          parentKey: visitorKey,
          parentPath: currentPath
        });
      }
    }
  }
  return true;
}

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class CommandContext {
  constructor(context, comment, command, matches) {
    /**
     * The ESLint RuleContext
     */
    __publicField(this, "context");
    /**
     * The comment node that triggered the command
     */
    __publicField(this, "comment");
    /**
     * Command that triggered the context
     */
    __publicField(this, "command");
    /**
     * Alias for `this.context.sourceCode`
     */
    __publicField(this, "source");
    /**
     * Regexp matches
     */
    __publicField(this, "matches");
    this.context = context;
    this.comment = comment;
    this.command = command;
    this.source = context.sourceCode;
    this.matches = matches;
  }
  /**
   * A shorthand of `this.context.sourceCode.getText(node)`
   *
   * When `node` is `null` or `undefined`, it returns an empty string
   */
  getTextOf(node) {
    if (!node)
      return "";
    if (Array.isArray(node))
      return this.context.sourceCode.text.slice(node[0], node[1]);
    return this.context.sourceCode.getText(node);
  }
  /**
   * Report an ESLint error on the triggering comment, without fix
   */
  reportError(message, ...causes) {
    this.context.report({
      loc: this.comment.loc,
      messageId: "command-error",
      data: {
        command: this.command.name,
        message
      }
    });
    for (const cause of causes) {
      const { message: message2, ...pos } = cause;
      this.context.report(
        {
          ...pos,
          messageId: "command-error-cause",
          data: {
            command: this.command.name,
            message: message2
          }
        }
      );
    }
  }
  /**
   * Report an ESLint error.
   * Different from normal `context.report` as that it requires `message` instead of `messageId`.
   */
  report(descriptor) {
    const { message, ...report } = descriptor;
    this.context.report({
      ...report,
      messageId: "command-fix",
      data: {
        command: this.command.name,
        message,
        ...report.data
      }
    });
  }
  /**
   * Utility to traverse the AST starting from a node
   */
  traverse(node, cb) {
    return traverse(this.context, node, cb);
  }
  /**
   * Report an ESLint error that removes the triggering comment
   */
  removeComment() {
    this.context.report({
      loc: this.comment.loc,
      messageId: "command-removal",
      data: {
        command: this.command.name
      },
      fix: (fixer) => {
        const lastToken = this.context.sourceCode.getTokenBefore(
          this.comment,
          { includeComments: true }
        )?.range[1];
        let lineStart = this.context.sourceCode.getIndexFromLoc({
          line: this.comment.loc.start.line,
          column: 0
        }) - 1;
        if (lastToken != null)
          lineStart = Math.max(lastToken, lineStart);
        return fixer.removeRange([
          lineStart,
          this.comment.range[1]
        ]);
      }
    });
  }
  // Implementation
  findNodeBelow(...args) {
    let options;
    if (typeof args[0] === "string")
      options = { types: args };
    else if (typeof args[0] === "function")
      options = { filter: args[0] };
    else
      options = args[0];
    const {
      shallow = false,
      findAll = false
    } = options;
    const tokenBelow = this.context.sourceCode.getTokenAfter(this.comment);
    if (!tokenBelow)
      return;
    const nodeBelow = this.context.sourceCode.getNodeByRangeIndex(tokenBelow.range[1]);
    if (!nodeBelow)
      return;
    const result = [];
    let target = nodeBelow;
    while (target.parent && target.parent.loc.start.line === nodeBelow.loc.start.line)
      target = target.parent;
    const filter = options.filter ? options.filter : (node) => options.types.includes(node.type);
    this.traverse(target, (path) => {
      if (path.node.loc.start.line !== nodeBelow.loc.start.line)
        return STOP;
      if (filter(path.node)) {
        result.push(path.node);
        if (!findAll)
          return STOP;
        if (shallow)
          return SKIP;
      }
    });
    return findAll ? result : result[0];
  }
}

function createRuleWithCommands(commands) {
  return createEslintRule({
    name: "command",
    meta: {
      type: "problem",
      docs: {
        description: "Comment-as-command for one-off codemod with ESLint"
      },
      fixable: "code",
      schema: [],
      messages: {
        "command-error": "[{{command}}] error: {{message}}",
        "command-error-cause": "[{{command}}] error cause: {{message}}",
        "command-fix": "[{{command}}] fix: {{message}}",
        "command-removal": "[{{command}}] remove comment after use"
      }
    },
    defaultOptions: [],
    create: (context) => {
      const sc = context.sourceCode;
      const comments = sc.getAllComments();
      for (const comment of comments) {
        if (comment.type !== "Line")
          continue;
        const commandRaw = comment.value.trim();
        for (const command of commands) {
          const matches = commandRaw.match(command.match);
          if (matches) {
            command.action(new CommandContext(context, comment, command, matches));
            continue;
          }
        }
      }
      return {};
    }
  });
}
const command = /* @__PURE__ */ createRuleWithCommands(index.builtinCommands);

function createPluginWithCommands(options = {}) {
  const {
    name = "command"
  } = options;
  return {
    meta: {
      name,
      version
    },
    rules: {
      command
    }
  };
}

const defaultPlugin = createPluginWithCommands();

exports.createPluginWithCommands = createPluginWithCommands;
exports.defaultPlugin = defaultPlugin;
